@@ -4,9 +4,56 @@ import logging
 import select
 import subprocess
 import threading
+import types
 from subprocess import DEVNULL, PIPE
 
 
+def polled_pipe_logger(self):
+    # https://stackoverflow.com/a/49500729
+    lines = b""
+    poller = select.poll()
+    poller.register(self.pipe, select.POLLHUP | select.POLLIN)
+    while True:
+        for descriptor, mask in poller.poll(-1):
+            if descriptor == self.pipe.fileno():
+                if mask & select.POLLIN:
+                    r = self.pipe.read()
+                    lines += r
+                    while b"\n" in lines:
+                        line, lines = lines.split(b"\n", 1)
+                        self.logger.info("%s", line.decode())
+                elif mask & select.POLLHUP:
+                    self.logger.debug("Hang up")
+                    return
+                else:
+                    self.logger.error("Unknown mask: 0x%x", mask)
+            else:
+                self.logger.error("Unknown descriptor: 0x%x", descriptor)
+
+
+def default_pipe_logger(self):
+    # https://stackoverflow.com/a/49500729
+    lines = b""
+    while True:
+        r = self.pipe.read()
+        lines += r
+        while b"\n" in lines:
+            line, lines = lines.split(b"\n", 1)
+            self.logger.info("%s", line.decode())
+        if not r:
+            self.logger.info("%s", lines.decode())
+            return
+
+
+def pipe_logger_factory():
+    if hasattr(select, "poll"):
+        logging.info('Using Polled pipe logger')
+        return polled_pipe_logger
+    else:
+        logging.info('Using default pipe logger')
+        return default_pipe_logger
+
+
 class PipeFollower(threading.Thread):
     def __init__(self, name, pname, pipe):
         threading.Thread.__init__(
@@ -16,32 +63,14 @@ class PipeFollower(threading.Thread):
         self.logger = logging.getLogger(
             ".".join([self.__class__.__name__, name, pname])
         )
+        self.pipe_logger = types.MethodType(
+            pipe_logger_factory(), self, PipeFollower
+        )
 
     def run(self):
         try:
             if hasattr(self.pipe, "read"):
-                lines = b""
-                # https://stackoverflow.com/a/49500729
-                poller = select.poll()
-                poller.register(self.pipe, select.POLLHUP | select.POLLIN)
-                while True:
-                    for descriptor, mask in poller.poll(-1):
-                        if descriptor == self.pipe.fileno():
-                            if mask & select.POLLIN:
-                                r = self.pipe.read()
-                                lines += r
-                                while b"\n" in lines:
-                                    line, lines = lines.split(b"\n", 1)
-                                    self.logger.info("%s", line.decode())
-                            elif mask & select.POLLHUP:
-                                self.logger.debug("Hang up")
-                                return
-                            else:
-                                self.logger.error("Unknown mask: 0x%x", mask)
-                        else:
-                            self.logger.error(
-                                "Unknown descriptor: 0x%x", descriptor
-                            )
+                return self.pipe_logger()
             else:
                 with open(self.pipe, "rt", buffering=1) as read:
                     self.pipe = read
